@@ -107,6 +107,7 @@ class APOS:
             apos = fmodpy.fimport(_source_code, blas=True,
                                   lapack=True, omp=True, wrap=True,
                                   verbose=False, output_dir=_this_dir,
+                                  dependencies=["matrix_operations.f90", "sort_and_select.f90"]
             )
             # Store the Fortran module as an attribute.
             self.APOS = apos.apos
@@ -227,7 +228,7 @@ class APOS:
 
 
     # Convert all inputs to the APOS model into the expected numpy format.
-    def _to_array(self, y, yi, x, xi, ax, axi, sizes):
+    def _to_array(self, ax, axi, sizes, x, xi, y, yi):
         # Get the number of inputs.
         if   (y  is not None): nm = len(y)
         elif (yi is not None): nm = len(yi)
@@ -314,8 +315,8 @@ class APOS:
 
 
     # Fit this model.
-    def fit(self, x=None, y=None, yi=None, xi=None, ax=None, axi=None,
-            sizes=None, new_model=False, **kwargs):
+    def fit(self, ax=None, axi=None, sizes=None, x=None, xi=None,
+            y=None, yi=None, yw=None, new_model=False, **kwargs):
         # Ensure that 'y' values were provided.
         assert ((y is not None) or (yi is not None)), "APOS.fit requires 'y' or 'yi' values, but neitherwere provided (use keyword argument 'y=<values>' or 'yi=<values>')."
         # Make sure that 'sizes' were provided for apositional (aggregate) inputs.
@@ -323,8 +324,14 @@ class APOS:
             assert (sizes is not None), "APOS.fit requires 'sizes' to be provided for apositional input sets (ax and axi)."
         # Get all inputs as arrays.
         nm, na, mdn, mne, mdo, adn, ane, yne, y, x, xi, ax, axi, sizes = (
-            self._to_array(y, yi, x, xi, ax, axi, sizes)
+            self._to_array(ax, axi, sizes, x, xi, y, yi)
         )
+        # Convert yw to a numpy array (if it is not already).
+        if (yw is None):
+            yw = np.zeros((nm,0), dtype="float32", order="C")
+        else:
+            yw = np.asarray(np.asarray(yw, dtype="float32").reshape((nm,-1)), order="C")
+        assert (yw.shape[1] in {0, 1, mdo}), f"Weights for points 'yw' {yw.shape} must have 1 column{' or '+str(mdo)+' columns' if (mdo > 0) else ''}."
         # Configure this model if requested (or not already done).
         if (new_model or (self.config is None)):
             # Ensure that the config is compatible with the data.
@@ -340,6 +347,12 @@ class APOS:
                 kwargs["mdo"] = 0
                 kwargs["mns"] = 0
             self._init_model(**kwargs)
+        else:
+            # Set any configuration keyword arguments.
+            for n in ({n for (n,t) in self.config._fields_} & set(kwargs)):
+                if (kwargs[n] is not None):
+                    setattr(self.config, n, kwargs[n])
+            
         # If there are integer embeddings, expand "x" and "ax" to have space to hold those embeddings.
         if (self.config.ade > 0):
             _ax = np.zeros((ax.shape[0],ax.shape[1]+self.config.ade), dtype="float32", order="C")
@@ -366,7 +379,7 @@ class APOS:
         # Minimize the mean squared error.
         self.record = np.zeros((steps,6), dtype="float32", order="C")
         result = self.APOS.minimize_mse(self.config, self.model, rwork, iwork,
-                                        ax.T, axi.T, sizes, x.T, xi.T, y.T, 
+                                        ax.T, axi.T, sizes, x.T, xi.T, y.T, yw.T,
                                         steps=steps, record=self.record.T)
         assert (result[-1] == 0), f"APOS.minimize_mse returned nonzero exit code {result[-1]}."
         # Copy the updated values back into the input arrays (for transparency).
@@ -391,7 +404,7 @@ class APOS:
             assert (sizes is not None), "APOS.predict requires 'sizes' to be provided for apositional input sets (ax and axi)."
         # Make sure that all inputs are numpy arrays.
         nm, na, mdn, mne, mdo, adn, ane, yne, _, x, xi, ax, axi, sizes = (
-            self._to_array(None, None, x, xi, ax, axi, sizes)
+            self._to_array(ax, axi, sizes, x, xi, None, None)
         )
         # Embed the inputs into the purely positional form.
         ade = self.config.ade
@@ -527,39 +540,42 @@ if __name__ == "__main__":
         x, y = x[:,0], x[:,1]
         return (3*x + np.cos(8*x)/2 + np.sin(5*y))
 
+
     n = 100
     seed = 2
-    state_dim = 16
+    state_dim = 40
     num_states = 4
     steps = 1000
     num_threads = None
     np.random.seed(seed)
 
+
+    TEST_WEIGHTING = False
     TEST_FIT_SIZE = False
     TEST_SAVE_LOAD = True
     TEST_INT_INPUT = False
     TEST_APOSITIONAL = False
-    TEST_VARIED_SIZE = False
+    TEST_LARGE_MODEL = False
     SHOW_VISUALS = True
 
 
     if TEST_FIT_SIZE:
         # Apositional model settings.
         dim_a_numeric = 4
-        dim_a_embedding = 32
+        dim_a_embedding = None
         num_a_embeddings = 32
-        dim_a_state = 128
-        num_a_states = 1
-        dim_a_out = 64
+        dim_a_state = 8
+        num_a_states = 2
+        dim_a_out = None
         # Model settings.
         dim_m_numeric = 8
-        dim_m_embedding = 0
-        num_m_embeddings = 0
+        dim_m_embedding = 32
+        num_m_embeddings = 32
         dim_m_state = 64
         num_m_states = 8
         dim_m_out = 1
         # Number of points going into each model.
-        na = 10000000
+        na = 100000000
         nm = 100000
         num_threads = 100
         # Initialize the model and its fit configuration.
@@ -581,6 +597,7 @@ if __name__ == "__main__":
             def __getattr__(self, *args, **kwargs):
                 return lambda *args, **kwargs: None
 
+
     if TEST_SAVE_LOAD:
         # Try saving an untrained model.
         m = APOS()
@@ -592,6 +609,7 @@ if __name__ == "__main__":
         from util.approximate import PLRM
         m = APOS(mdn=2, mds=state_dim, mns=num_states, mdo=1, seed=seed,
                  num_threads=num_threads, steps=steps, 
+                 orthogonalizing_step_frequency=10,
                  ) # discontinuity=-1000.0) # initial_step=0.01)
         print("Initialized model:")
         print(m)
@@ -600,12 +618,18 @@ if __name__ == "__main__":
         x = np.asarray(well_spaced_box(n, 2), dtype="float32", order="C")
         # x[:,0] /= 2
         y = f(x).astype("float32")
+        # Construct weights.
+        yw = np.random.random(size=n) ** 5
+        yw = np.where(yw > 0.5, yw*2, 0.001)
+        if (not TEST_WEIGHTING): yw[:] = 1.0
         # Fit the model.
-        m.fit(x.copy(), y.copy())
+        m.fit(x=x.copy(), y=y.copy(), yw=yw)
         # Add the data and the surface of the model to the plot.
         p = Plot()
         x_min_max = np.asarray([x.min(axis=0), x.max(axis=0)]).T
-        p.add("Data", *x.T, y)
+        p.add("Data", *x[yw > 0.5].T, y[yw > 0.5])
+        if (TEST_WEIGHTING):
+            p.add("Data (low weight)", *x[yw <= 0.5].T, y[yw <= 0.5], color=3)
         # p.add("Normalized data", *x_fit.T, y_fit)
         p.add_func("Fit", m, *x_min_max, vectorized=True)
         # Try saving the trained model and applying it after loading.
@@ -677,7 +701,7 @@ if __name__ == "__main__":
                  ane=len(np.unique(axi.flatten())), mne=len(np.unique(all_xi.flatten())),
                  num_threads=num_threads, seed=seed)
         m.fit(ax=ax.copy(), axi=axi, sizes=sizes, xi=all_xi, y=all_y.copy(), 
-              steps=steps, num_threads=num_threads, seed=seed)
+              steps=steps, num_threads=num_threads, seed=seed, basis_replacement=False)
         # Create an evaluation set that evaluates the model that was built over two differnt functions.
         xi1 = np.ones((len(x),1),dtype="int32")
         ax = x.reshape((-1,1)).copy()
@@ -704,6 +728,44 @@ if __name__ == "__main__":
         p.show(show=False)
 
 
+    if (TEST_LARGE_MODEL):
+        # Data size.
+        nm = 100000
+        na = 6000000
+        adn = 10
+        ane = 10
+        mdn = 10
+        mne = 10
+        mdo = 10
+        # Data to fit.
+        ax = np.random.random(size=(na,adn)).astype("float32")
+        axi = np.random.randint(0,ane,size=(na,1)).astype("float32")
+        x = np.random.random(size=(nm,mdn)).astype("float32")
+        xi = np.random.randint(0,mne,size=(nm,1)).astype("float32")
+        y = np.random.random(size=(nm,mdo)).astype("float32")
+        # Generate random starting indices for all the sizes and
+        #   convert those starting indices into sizes of batches.
+        sizes = np.arange(na, dtype="int32")
+        np.random.shuffle(sizes)
+        sizes = sizes[:nm]
+        sizes[0] = 0
+        sizes.sort()
+        sizes[:-1] = sizes[1:] - sizes[:-1]
+        sizes[-1] = na - sizes[-1]
+        # Model settings.
+        ans = 4
+        ads = 64
+        mns = 8
+        mds = 64
+        steps = 11
+        num_threads = 20
+        # Fit model.
+        m = APOS(adn=adn, ane=ane, mdn=mdn, mne=mne, mdo=mdo,
+                 ans=ans, ads=ads, mns=mns, mds=mds)
+        m.fit(ax=ax, axi=axi, sizes=sizes, x=x, xi=xi, y=y,
+              steps=steps, num_threads=num_threads)
+
+
     # Generate a visual of the loss function.
     if (SHOW_VISUALS and (len(getattr(globals().get("m",None), "record", [])) > 0)):
         print()
@@ -721,87 +783,3 @@ if __name__ == "__main__":
             p.add("Grad utilization", step_indices, record[:i,5], color=6, mode="lines", frame=i)
         p.show(append=True, show=True, y_range=[-.2, 1.2])
     print("", "done.", flush=True)
-
-
-    if TEST_VARIED_SIZE:
-        print("Creating data..")
-        for test in range(100):
-            print("sizes test: ", test, end="\r")
-            sizes = np.random.randint(5,20,size=(10))
-            na = sizes.sum()
-            nm = sizes.size
-            ax = np.random.random(size=(na,2))
-            x = well_spaced_box(nm, 2)
-            y = f(x)
-            start = 0
-            for i in range(len(sizes)):
-                end = start + sizes[i]
-                y[i] += ax[start:end].max()
-                start = end
-            # Fit a model.
-            m = APOS(seed=seed, num_threads=num_threads, steps=1)
-            m.fit(x=x.copy(), y=y.copy(), ax=ax.copy(), sizes=sizes)
-
-
-# 2022-04-03 15:03:09
-# 
-        #########################################################################
-        # # Generate visual of the embeddings.                                  #
-        # _ = m.unpack()                                                        #
-        # embeddings = _.m_embeddings.T                                         #
-        # output_vecs = _.m_output_vecs.T                                       #
-        # print()                                                               #
-        # print("Embeddings:")                                                  #
-        # print(embeddings)                                                     #
-        # print()                                                               #
-        # print("Output vecs:")                                                 #
-        # print(output_vecs)                                                    #
-        # print()                                                               #
-        #                                                                       #
-        # p = Plot("Embeddings")                                                #
-        # # Reduce the dimension to 3 if it is higher (using singular vectors). #
-        # if (embeddings.shape[1] > 3):                                         #
-        #     from tlux.math import svd                                         #
-        #     _, projection = svd(embeddings.copy())                            #
-        #     embeddings = np.matmul(embeddings, projection[:3].T)              #
-        # # Raise the dimension from 1 to 2.                                    #
-        # elif (embeddings.shape[1] == 1):                                      #
-        #     embeddings = np.asarray([[v,0] for v in embeddings.flatten()])    #
-        # # Add the embedding vectors to the plot.                              #
-        # for vec in embeddings:                                                #
-        #     p.add(str(vec), *[[0,v] for v in vec], mode="lines")              #
-        # p.show(append=True, show=False)                                       #
-        #########################################################################
-
-
-# 2022-04-03 19:43:37
-# 
-        ###########################################################################################
-        # if False:                                                                               #
-        #     m = APOS(mdn=0, adn=ax.shape[1], ado=2, mdo=0, #all_y.shape[1],                     #
-        #              ads=state_dim, ans=num_states, # mds=state_dim, mns=num_states,            #
-        #              ane=len(np.unique(axi.flatten())), # mne=len(np.unique(all_xi.flatten())), #
-        #              num_threads=num_threads, seed=seed)                                        #
-        #     fit_ax = np.array(ax, dtype="float32", order="C")                                   #
-        #     fit_y = np.array(all_x, dtype="float32", order="C")                                 #
-        #     print("fit_y.mean(axis=0): ", fit_y.mean(axis=0))                                   #
-        #     print()                                                                             #
-        #     print(m.config_str())                                                               #
-        #     print()                                                                             #
-        #     m.fit(ax=fit_ax, axi=axi, sizes=sizes, y=fit_y,                                     #
-        #           steps=1000, num_threads=num_threads, seed=seed)                               #
-        #     fx = m(ax=ax.copy(), axi=axi, sizes=sizes)                                          #
-        #     print("fit_y.mean(axis=0): ", fit_y.mean(axis=0))                                   #
-        #     print("m.unpack().ay_shift: ", m.unpack().ay_shift)                                 #
-        #     print(" m.unpack().y_shift: ", m.unpack().y_shift)                                  #
-        #     print()                                                                             #
-        #     print("all_x: ")                                                                    #
-        #     print(all_x)                                                                        #
-        #     print()                                                                             #
-        #     print("fx: ")                                                                       #
-        #     print(fx)                                                                           #
-        #     print()                                                                             #
-        #     print("all_x - fx: ")                                                               #
-        #     print(all_x - fx)                                                                   #
-        # else:                                                                                   #
-        ###########################################################################################
